/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.testcase;

import java.lang.ref.SoftReference;
import java.util.LinkedList;
import java.util.List;

import javax.swing.AbstractListModel;

import com.eviware.soapui.model.testsuite.TestStepResult;

/**
 * ListModel for the TestCaseLog
 * 
 * @author ole.matzura
 */

public class TestCaseLogModel extends AbstractListModel
{
	private List<Object> items = new LinkedList<Object>();
	private List<SoftReference<TestStepResult>> results = new LinkedList<SoftReference<TestStepResult>>();
	private int stepCount;
	private int maxSize = 500;
	private boolean discardRemoved;

	public void addText(String msg)
	{
		synchronized( this )
		{
			items.add(msg);
			results.add(null);
			fireIntervalAdded(this, items.size() - 1, items.size()-1 );
			
			enforceMaxSize();
		}
	}

	private void enforceMaxSize()
	{
		while( items.size() > maxSize )
		{
			items.remove( 0 );
			SoftReference<TestStepResult> result = results.remove( 0 );
			if( discardRemoved && result != null && result.get() != null && !result.get().isDiscarded())
				result.get().discard();
			
			fireIntervalAdded(this, 0, 0 );
		}
	}

	public void addTestStepResult(TestStepResult result)
	{
		synchronized( this )
		{
			stepCount++;
	
			int size = items.size();
			items.add("Step " + stepCount + " [" + result.getTestStep().getName() + "] " + result.getStatus() + ": took " + result.getTimeTaken() + " ms");
			SoftReference<TestStepResult> ref = new SoftReference<TestStepResult>(result);
			results.add( ref);
			for (String msg : result.getMessages())
			{ 
				items.add(" -> " + msg);
				results.add(ref);
			}
	
			fireIntervalAdded(this, size, items.size()-1 );
			enforceMaxSize();
		}
	}

	public void clear()
	{
		synchronized( this )
		{
			int sz = getSize();
			items.clear();
			results.clear();
			stepCount = 0;
			fireIntervalRemoved(this, 0, sz);
		}
	}

	public int getMaxSize()
	{
		return maxSize;
	}

	public synchronized void setMaxSize( int maxSize )
	{
		this.maxSize = maxSize;
		enforceMaxSize();
	}

	public boolean isDiscardRemoved()
	{
		return discardRemoved;
	}

	public void setDiscardRemoved( boolean discardRemoved )
	{
		this.discardRemoved = discardRemoved;
	}

	public int getSize()
	{
		synchronized( this )
		{
			return items.size();
		}
	}

	public Object getElementAt(int arg0)
	{
		try
		{
			return items.get(arg0);
		}
		catch (Throwable e)
		{
			return null;
		}
	}

	public TestStepResult getResultAt(int index)
	{
		if( index >= results.size())
			return null;
		
		SoftReference<TestStepResult> result = results.get(index);
		return result == null ? null : result.get();
	}

	public void setStepIndex( int i )
	{
		stepCount = i;
	}
}